# %{lua: -- file: vdb.lua -- #

-- Functions I frequently use in rpm spec files.

-- Die flag name. Die flag is environment variable.
-- If die flag is set (to non-empty value),
-- %setup sriplet will stop build.
local _die_ = "_rpm_die_" .. math.random( 1000 ) .. "_"
posix.setenv( _die_, "" )

subpackages = {}

-- Escape % characters, return string with all % characters doubled.
function escape( str )
    return string.gsub( str, "%%", "%%%%" )
end

-- Quote string for using in shell scripts.
function quote( str )
    return "'" .. string.gsub( str, "'", "'\\''" ) .. "'"
end

-- Perl-like map: apply func to the each element of the list.
function map( func, list )
    local result = {}
    for i, v in ipairs( list ) do
        table.insert( result, func( v ) )
    end
    return result
end

-- Perl-like join: concatenate all list elements.
function join( sep, list )
    return table.concat( list, sep )
end

-- Perl-like uc: convert string to upper case.
uc = string.upper

-- Define rpm macro.
function define( name, value )
    rpm.define( name .. " " .. value )
    return value
end

-- Expand rpm macro(s).
function expand( str )
    return rpm.expand( str )
end

-- Get value of rpm macro.
function value( name )
    return expand( "%{" .. escape( name ) .. "}" )
end

-- Print a message to stderr.
function say( message )
    expand( "%{echo:" .. escape( message ) .. "\n}" )
    return message
end

-- Print a error to stderr.
-- Building will be stopped in the nearest %prologue.
function die( message )
    if not message or message == "" then
        message = "(no error message provided)"
    end
    expand( "%{error:" .. escape( message ) .. "}" )
    -- I wish rpmbuild stops here, but it continues. :-(
    -- So let us save error in the environment variable.
    -- The next %prologue will check the variable and stop execution.
    posix.setenv( _die_, message )
    return message
end

-- Returns true if file exists.
function exist( file )
    return not not posix.stat( file )
end

-- Rename file or directory.
function rename( old, new )
    say( "+ mv " .. old .. " " .. new )
    local ok, error = os.rename( old, new )
    if not ok then
        die( "mv: " .. error )
    end
    return ok
end

-- Execute shell command.
function run( command )
    say( "+ " .. command )
    local ok, error = os.execute( command )
    if not ok then
        die( command .. ": " .. error )
    end
    return ok
end

-- Return n first lines of the file.
function head( file, n )
    local lines = {}
    local i = 0
    for line in io.lines( file ) do
        lines[ i ] = line
        i = i + 1
        if i == n then
            break
        end
    end
    return lines
end

function banner( phase )
    return ": ---------- " .. phase .. " ---------- : +\n"
end

-- Prologue to execute in the beginnning of every scriptlet:
prologue = "set -e; [ -n \"$BASH_VERSION\" ] && set -o pipefail;\n"
define( "prologue", prologue )

function prep()
    -- Someone can run `rpmbuild -bb name.spec` and build rpm files. However, exact name of built
    -- rpm is not known to the caller because name contains version, release, and architecture --
    -- these parameters are encoded into spec and not easily available outside of the spec file.
    -- Let us write them out, so the caller can source the file and construct rpm name, e. g.
    -- "$NAME-$VERSION-$RELEASE.$ARCH.rpm".
    local name = os.getenv( "SPEC_RC_FILE" )
    if name then
        local file, error = io.open( name, "w" )
        if file then
            file:write(
                "NAME="    .. quote( expand( "%{name}"                              ) ) .. "\n" ..
                "VERSION=" .. quote( expand( "%{version}"                           ) ) .. "\n" ..
                "RELEASE=" .. quote( expand( "%{release}"                           ) ) .. "\n" ..
                "ARCH="    .. quote( expand( "%{?buildarch}%{!?buildarch:%{_arch}}" ) ) .. "\n" ..
                "RPMS="    .. quote( expand( "%{_rpmdir}"                           ) ) .. "\n" ..
                "SRPMS="   .. quote( expand( "%{_srcrpmdir}"                        ) ) .. "\n" ..
                "SUBPACKAGES=(" .. join( " ", map( quote, subpackages ) ) .. ")\n" ..
                "\n"
            )
            file:close()
        else
            -- Lua's error already contains file name.
            die( "Failed to write rc file: " .. error )
        end
    end
    print( banner( "PREP" ), prologue )
    -- All the lua code embedded with %{lua: ...} to the spec file
    -- is executed before the first scriplet (namely, %prep) starts working.
    -- If lua code set _die_ flag, let us stop build.
    print( "[ -z \"${" .. _die_ .. "}\" ];" )
end

function build()
    print( banner( "BUILD" ), prologue )
end

function check()
    print( banner( "CHECK" ), prologue )
end

function install()
    print( banner( "INSTALL" ), prologue )
end

-- Redefine %prep, %build, %check and %install macros:
-- Let's add prologue to the every build-time scriptlet.
local phases = { "prep" }
--~ local phases = { "prep", "build", "check", "install" }
for i, phase in ipairs( phases ) do
    define( phase, "%%" .. phase .. " \\\n" .. "%{lua: " .. phase .. "();  }\n" )
end

-- The same for install-time scriplets.
local phases = {
    "pretrans",
    "pre",
    "post",
    "triggerin",
    "triggerun",
    "preun",
    "postun",
    "triggerpostun",
    "posttrans",
}
for i, phase in ipairs( phases ) do
    -- If -p option is present, do not add prologue.
    define( phase .. "(n:p:)", "%%" .. phase .. " %** \\\n" .. "%{!-p:%prologue}\n" )
end

-- Play the same trick to collect all subpackage names:
define(
    -- %package macro may have -n option. If -n is present, it defines full name of the subpackage.
    -- Oterwise subpackage name is constructed from package name and the first macro argument.
    "package(n:)",
    "%%package %** \\\n" ..
        "%{lua: table.insert( subpackages, expand( \"%{-n*}%{!-n:%name-%1}\" ) ) }\n"
)

define( "_licensedir", "%{_datadir}/licenses" )
define( "_fontdir",    "%{_datadir}/fonts" )

-- } # end of file # --
